#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import tkinter as tk
from tkinter import messagebox
import random

# Constants
PLAYER = "X"
AI = "O"
EMPTY = " "

class TicTacToe:
    def __init__(self):
        self.window = tk.Tk()
        self.window.title("Tic-Tac-Toe")
        self.window.resizable(False, False)
        self.difficulty = "Unbeatable"  # Default difficulty
        self.create_menu()

    def create_menu(self):
        self.menu_frame = tk.Frame(self.window, bg='lightblue')
        self.menu_frame.pack(pady=50)

        title_label = tk.Label(self.menu_frame, text="Welcome to Tic-Tac-Toe", font=("Arial", 24), bg='lightblue')
        title_label.pack(pady=10)

        instructions = ("Instructions:\n"
                        "- The game is played on a grid that's 3 squares by 3 squares.\n"
                        "- You are X, your opponent (the computer) is O.\n"
                        "- Players take turns putting their marks in empty squares.\n"
                        "- The first player to get 3 of their marks in a row (up, down, across, or diagonally) wins.\n"
                        "- When all 9 squares are full, the game is over.\n"
                        "  If no player has 3 marks in a row, the game ends in a draw.")

        instruction_label = tk.Label(self.menu_frame, text=instructions, font=("Arial", 14), bg='lightblue', justify=tk.LEFT)
        instruction_label.pack(pady=20)

        start_button = tk.Button(self.menu_frame, text="Start Game", command=self.start_game, font=("Arial", 18), bg='green', fg='white')
        start_button.pack(pady=10)

        exit_button = tk.Button(self.menu_frame, text="Exit", command=self.window.quit, font=("Arial", 18), bg='red', fg='white')
        exit_button.pack(pady=10)

        self.window.mainloop()

    def start_game(self):
        if hasattr(self, 'menu_frame'):
            self.menu_frame.pack_forget()
        if hasattr(self, 'board_frame'):
            self.board_frame.destroy()
        if hasattr(self, 'status_label'):
            self.status_label.destroy()
        if hasattr(self, 'restart_button'):
            self.restart_button.destroy()

        self.board = [[EMPTY for _ in range(3)] for _ in range(3)]
        self.current_player = PLAYER
        self.buttons = [[None for _ in range(3)] for _ in range(3)]
        self.create_board()

    def create_board(self):
        self.board_frame = tk.Frame(self.window, bg='lightyellow')
        self.board_frame.pack()

        for i in range(3):
            for j in range(3):
                button = tk.Button(self.board_frame, text="", width=10, height=3, font=("Arial", 24),
                                   command=lambda row=i, col=j: self.handle_click(row, col), bg='lightgray')
                button.grid(row=i, column=j, padx=5, pady=5)
                self.buttons[i][j] = button

        self.status_label = tk.Label(self.window, text=f"{self.current_player}'s turn", font=("Arial", 18), bg='lightyellow')
        self.status_label.pack(pady=10)

    def handle_click(self, row, col):
        if self.buttons[row][col]["text"] == "" and self.current_player == PLAYER:
            self.buttons[row][col]["text"] = PLAYER
            self.buttons[row][col].config(bg='lightblue')
            self.board[row][col] = PLAYER
            if self.check_winner(PLAYER):
                self.game_over(f"Player {PLAYER} wins!")
            elif self.is_draw():
                self.game_over("It's a draw!")
            else:
                self.current_player = AI
                self.status_label.config(text=f"{self.current_player}'s turn")
                self.window.after(500, self.ai_move)

    def ai_move(self):
        if self.difficulty == "Random":
            self.random_move()
        elif self.difficulty == "Basic":
            self.basic_move()
        elif self.difficulty == "Unbeatable":
            self.unbeatable_move()

        if self.check_winner(AI):
            self.game_over(f"Player {AI} wins!")
        elif self.is_draw():
            self.game_over("It's a draw!")
        else:
            self.current_player = PLAYER
            self.status_label.config(text=f"{self.current_player}'s turn")

    def random_move(self):
        empty_cells = [(i, j) for i in range(3) for j in range(3) if self.board[i][j] == EMPTY]
        row, col = random.choice(empty_cells)
        self.make_move(row, col, AI)

    def basic_move(self):
        for row in range(3):
            for col in range(3):
                if self.board[row][col] == EMPTY:
                    self.make_move(row, col, AI)
                    if self.check_winner(AI):
                        return
                    else:
                        self.board[row][col] = EMPTY
                        self.buttons[row][col]["text"] = ""

        self.random_move()

    def unbeatable_move(self):
        best_score = float('-inf')
        best_move = None

        for row in range(3):
            for col in range(3):
                if self.board[row][col] == EMPTY:
                    self.board[row][col] = AI
                    score = self.minimax(self.board, 0, False)
                    self.board[row][col] = EMPTY
                    if score > best_score:
                        best_score = score
                        best_move = (row, col)

        self.make_move(best_move[0], best_move[1], AI)

    def minimax(self, board, depth, is_maximizing):
        if self.check_winner(AI):
            return 1
        elif self.check_winner(PLAYER):
            return -1
        elif self.is_draw():
            return 0

        if is_maximizing:
            best_score = float('-inf')
            for row in range(3):
                for col in range(3):
                    if board[row][col] == EMPTY:
                        board[row][col] = AI
                        score = self.minimax(board, depth + 1, False)
                        board[row][col] = EMPTY
                        best_score = max(score, best_score)
            return best_score
        else:
            best_score = float('inf')
            for row in range(3):
                for col in range(3):
                    if board[row][col] == EMPTY:
                        board[row][col] = PLAYER
                        score = self.minimax(board, depth + 1, True)
                        board[row][col] = EMPTY
                        best_score = min(score, best_score)
            return best_score

    def make_move(self, row, col, player):
        self.buttons[row][col]["text"] = player
        self.board[row][col] = player
        if player == AI:
            self.buttons[row][col].config(bg='lightcoral')

    def check_winner(self, player):
        for row in range(3):
            if all([cell == player for cell in self.board[row]]):
                return True

        for col in range(3):
            if all([self.board[row][col] == player for row in range(3)]):
                return True

        if all([self.board[i][i] == player for i in range(3)]) or all([self.board[i][2 - i] == player for i in range(3)]):
            return True

        return False

    def is_draw(self):
        return all([cell != EMPTY for row in self.board for cell in row])

    def game_over(self, message):
        messagebox.showinfo("Game Over", message)
        self.restart_button = tk.Button(self.window, text="Restart Game", command=self.start_game, font=("Arial", 18), bg='blue', fg='white')
        self.restart_button.pack(pady=20)

if __name__ == "__main__":
    game = TicTacToe()

